import logging

from fingerprints.constants import PERSON, COMPANY, ANY
from fingerprints.constants import BRACKETED, WS
from fingerprints.data import COMPANY_TYPES, COMPANY_MAPPING, PERSON_PREFIX
from fingerprints.text import ensure_text, clean_strict

log = logging.getLogger(__name__)


def company_type_replacer(match):
    match = match.group(1)
    return COMPANY_MAPPING.get(match, match)


def generate(text, kind=ANY):
    text = ensure_text(text)
    if text is None:
        return

    # this needs to happen before the replacements
    text = text.lower()

    # try to remove personal prefix, such as Mr., Mrs.
    if kind in [ANY, PERSON]:
        text = PERSON_PREFIX.sub(WS, text)

    # remove any text in brackets
    text = BRACKETED.sub(WS, text)

    # super hard-core string scrubbing
    text = clean_strict(text)

    if kind in [ANY, COMPANY]:
        text = COMPANY_TYPES.sub(company_type_replacer, text)

    # final manicure, based on openrefine algo
    parts = [p for p in text.split(WS) if len(p)]
    text = WS.join(sorted(set(parts)))

    if not len(text):
        return None

    return text



import six
import logging
from unidecode import unidecode
from unicodedata import normalize, category

from fingerprints.constants import CATEGORIES, COLLAPSE, WS

log = logging.getLogger(__name__)


def ensure_text(text):
    """Make sure ``text`` is a useful snippet of text."""
    if text is None:
        return

    if not isinstance(text, six.text_type):
        try:
            text = six.text_type(text)
        except Exception as ex:
            log.exception(ex)

    if not isinstance(text, six.text_type):
        try:
            text = text.decode('utf-8')
        except Exception as ex:
            log.exception(ex)
            return None

    text = text.strip()
    if len(text) < 2:
        return None
    return text


def category_replace(text):
    """Replace unicode categories in the given text."""
    word = []
    for character in normalize('NFD', text):
        cat = category(character)[0]
        character = CATEGORIES.get(cat, character)
        if character is None:
            continue
        word.append(character)
    return ''.join(word)


def collapse(text):
    """Remove duplicate whitespaces."""
    return COLLAPSE.sub(WS, text)


def clean_strict(text):
    """Super-hardcore string scrubbing."""
    # transliterate to latin
    text = unidecode(text)
    # replace punctuation and symbols
    text = category_replace(six.text_type(text))
    # pad out for company type replacements
    text = ''.join((WS, collapse(text), WS))
    return text


def clean_replacement(text):
    """Apply scrubbing to replacement terms."""
    return clean_strict(six.text_type(text).lower())



import re

COLLAPSE = re.compile(r'\s+', re.U)
BRACKETED = re.compile(r'(\([^\(\)]*\)|\[[^\[\]]*\])')
WS = ' '

# Unicode character classes, see:
# http://www.fileformat.info/info/unicode/category/index.htm
CATEGORIES = {
    'C': None,
    'M': None,
    'Z': WS,
    'P': '',
    # 'S': WS
}

# Some hints for the normalizer:
PERSON = 1
COMPANY = 2
ADDRESS = 3
ANY = 4



import os
import re
import yaml

from fingerprints.text import clean_replacement


data_file = os.path.join(os.path.dirname(__file__), 'data.yaml')
with open(data_file, 'r') as fh:
    data = yaml.load(fh)

# Compile person prefixes into a regular expression.
person_prefixes = [p.lower().strip() for p in data.get('person_prefix')]
person_prefixes = '|'.join(person_prefixes)
person_prefixes = '^(%s)' % person_prefixes
PERSON_PREFIX = re.compile(person_prefixes, re.U)


# Compile company type mappings:
COMPANY_MAPPING = {}
company_types = list()
for clean_form, other_forms in data.get('company_types').items():
    clean_form = clean_replacement(clean_form)
    for form in other_forms:
        form = clean_replacement(form)
        if form not in company_types:
            COMPANY_MAPPING[form] = clean_form
            company_types.append(form)

company_types = '(%s)' % '|'.join(company_types)
COMPANY_TYPES = re.compile(company_types, re.U)



from setuptools import setup, find_packages


setup(
    name='fingerprints',
    version='0.1.3',
    description="A library to generate entity fingerprints.",
    long_description="",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ],
    keywords='names people companies normalisation',
    author='Friedrich Lindenberg',
    author_email='friedrich@pudo.org',
    url='http://github.com/pudo/fingerprint',
    license='MIT',
    packages=find_packages(exclude=['ez_setup', 'examples', 'test']),
    namespace_packages=[],
    package_data={
        '': ['fingerprint/data.yaml']
    },
    include_package_data=True,
    zip_safe=False,
    test_suite='nose.collector',
    install_requires=[
        'normality',
        'unidecode',
        'pyyaml',
        'six'
    ],
    tests_require=[
        'nose',
        'coverage',
        'wheel'
    ],
    entry_points={
    }
)
